package prompt.multiperspective.configuration;

import java.util.HashSet;
import java.util.Set;

/**
 * 
 * @author Andrea Burattin
 */
public class MultiperspectiveConfiguration {

	/**
	 * This enumeration identifies the possible activities that can be performed.
	 * 
	 * @author Andrea Burattin
	 */
	public enum ACTIVITY {
		
		CONTROL_FLOW_DISCOVERY("Control-Flow Perspective (required)"),
		ROLE_DISCOVERY("Social Perspective (Roles)"),
		//ADVANCED_SNA("Social Perspective (Advanced Social Network)"),
		GUARDS_DISCOVERY("Data Perspective (Guards)");
		
		private final String text;
		
		private ACTIVITY(final String text) {
			this.text = text;
		}
		
		@Override
		public String toString() {
			return text;
		}
	}
	
	/**
	 * This enumeration identifies the possible control-flow miners
	 * 
	 * @author Anna Kalenkova
	 *
	 */
	public enum CONTROL_FLOW_MINER {
		
		INDUCTIVE_MINER("Inductive Miner"),
		ALPHA_MINER("Alpha Miner"),
		EVOLUTIONARY_TREE_MINER("Evolutionary Tree Miner"),
		HEURISTIC_MINER("Heuristic Miner"),
		ILP_MINER("ILP Miner");
		
		private final String text;
		
		private CONTROL_FLOW_MINER(final String text) {
			this.text = text;
		}
		
		@Override
		public String toString() {
			return text;
		}
		
		public static CONTROL_FLOW_MINER getValueByText(String text) {
			for (CONTROL_FLOW_MINER miner : CONTROL_FLOW_MINER.values()) {
				if (miner.text == text) {
					return miner;
				}
			}
			return null;// not found
		}
	}
	
	/**
	 * Possible configurations for the control-flow miner
	 * 
	 * @author Anna Kalenkova
	 *
	 */
	public enum MINER_CONFIGURATION {
		
		SIMPLE("Simple configuration"),
		ADVANCED("Advanced configuration"),
		LOCALIZED_LOGS("Localized logs configuration");
		
		private final String text;
		
		private MINER_CONFIGURATION(final String text) {
			this.text = text;
		}
		
		@Override
		public String toString() {
			return text;
		}
	}


	private Set<ACTIVITY> activities = new HashSet<ACTIVITY>();
	private MINER_CONFIGURATION minerConfiguration = MINER_CONFIGURATION.SIMPLE;
	private CONTROL_FLOW_MINER controlFlowMiner = CONTROL_FLOW_MINER.INDUCTIVE_MINER;
	
	/**
	 * 
	 * @param activity
	 */
	public void addActivity(ACTIVITY activity) {
		activities.add(activity);
	}
	
	/**
	 * 
	 * @return
	 */
	public Set<ACTIVITY> getActivities() {
		return activities;
	}
	
	/**
	 * 
	 * @param controlFlowMiner
	 */
	public void setControlFlowMiner(CONTROL_FLOW_MINER controlFlowMiner) {
		this.controlFlowMiner = controlFlowMiner;
	}
	
	/**
	 * 
	 * @return
	 */
	public CONTROL_FLOW_MINER getControlFlowMiner() {
		return controlFlowMiner;
	}
	
	/**
	 * 
	 * @param simpleConfiguration
	 */
	public void setMinerConfiguration(MINER_CONFIGURATION minerConfiguration) {
		this.minerConfiguration = minerConfiguration;
	}
	
	/**
	 * 
	 * @return
	 */
	public MINER_CONFIGURATION getMinerConfiguration() {
		return minerConfiguration;
	}
}
