package prompt.multiperspective.configuration.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.io.IOException;
import java.io.InputStream;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ListCellRenderer;

import prompt.multiperspective.configuration.MultiperspectiveConfiguration;
import prompt.multiperspective.configuration.MultiperspectiveConfiguration.ACTIVITY;
import prompt.multiperspective.configuration.MultiperspectiveConfiguration.MINER_CONFIGURATION;
import prompt.multiperspective.utils.GUIUtils;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * 
 * @author Andrea Burattin
 */
public class SelectPerspectives extends JPanel {

	private static final long serialVersionUID = 8515466666580076870L;
	private static ImageIcon selectedIcon = null;
	private static ImageIcon notSelectedIcon = null;
	static {
		try {
			InputStream in = SelectPerspectives.class.getResourceAsStream("/prompt/multiperspective/resources/img/selected.png");
			selectedIcon = new ImageIcon(ImageIO.read(in));
			in = SelectPerspectives.class.getResourceAsStream("/prompt/multiperspective/resources/img/not_selected.png");
			notSelectedIcon = new ImageIcon(ImageIO.read(in));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	private JList<ACTIVITY> listSystems = new JList<ACTIVITY>();
	private JRadioButton simpleConfiguration = SlickerFactory.instance().createRadioButton("Simple configuration");
	private JRadioButton advancedConfiguration = SlickerFactory.instance().createRadioButton("Advanced configuration (for expert users)");
	private JRadioButton localizedLogsConfiguration = SlickerFactory.instance().createRadioButton("Advanced configuration "
			+ "with localization information (for expert users)");
	private MultiperspectiveConfiguration configuration = new MultiperspectiveConfiguration();
	
	public SelectPerspectives(MultiperspectiveConfiguration configuration) {
		this.configuration = configuration;
		init();
	}
	
	private void init() {
		DefaultListModel<ACTIVITY> listModel = new DefaultListModel<ACTIVITY>();
		for(ACTIVITY a : ACTIVITY.values()) {
			listModel.addElement(a);
		}
		
		listSystems.setModel(listModel);
		listSystems.setBackground(Color.lightGray);
		listSystems.setBorder(BorderFactory.createLineBorder(Color.darkGray, 1));
		listSystems.setCellRenderer(new ListCellRenderer<ACTIVITY>() {
			@Override
			public Component getListCellRendererComponent(
					JList<? extends ACTIVITY> list, ACTIVITY value, int index,
					boolean isSelected, boolean cellHasFocus) {
				JLabel l = new JLabel(value.toString());
				l.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
				if (isSelected) {
					l.setIcon(selectedIcon);
					l.setForeground(Color.white);
					l.setBackground(new Color(0, 140, 0));
					l.setOpaque(true);
				} else {
					l.setIcon(notSelectedIcon);
					l.setForeground(Color.darkGray);
					l.setOpaque(false);
				}
				return l;
			}
		});
		listSystems.setSelectionModel(new DefaultListSelectionModel() {
			private static final long serialVersionUID = -2930794960792504049L;
			@Override
			public void setSelectionInterval(int index0, int index1) {
				if (super.isSelectedIndex(index0)) {
					super.removeSelectionInterval(index0, index1);
				} else {
					super.addSelectionInterval(index0, index1);
				}
				
				if (!super.isSelectedIndex(0)) {
					super.addSelectionInterval(0, 0);
				}
			}
		});
		listSystems.setSelectedIndex(0);
		
		ButtonGroup options = new ButtonGroup();
		options.add(simpleConfiguration);
		options.add(advancedConfiguration);
		options.add(localizedLogsConfiguration);
		simpleConfiguration.setSelected(true);
		
		FormLayout layout = new FormLayout("fill:pref:grow");
		DefaultFormBuilder builder = new DefaultFormBuilder(layout);
		
		GUIUtils.configureSeparator(builder.appendSeparator("Select the Perspectives to Mine"));
		builder.append(Box.createVerticalStrut(10));
		builder.appendRow(RowSpec.decode("fill:pref:grow"));
		builder.append(listSystems);
		builder.append(Box.createVerticalStrut(20));
		GUIUtils.configureSeparator(builder.appendSeparator("Select the Configuration Type"));
		builder.append(Box.createVerticalStrut(10));
		builder.append(simpleConfiguration);
		builder.append(advancedConfiguration);
		builder.append(localizedLogsConfiguration);

		setLayout(new BorderLayout());
		add(builder.getPanel());
	}
	
	public void setSelectedSystems(Set<ACTIVITY> selection) {
		for (ACTIVITY s : selection) {
			listSystems.setSelectedValue(s, false);
		}
	}

	public MultiperspectiveConfiguration getConfiguration() {
		for(ACTIVITY s : listSystems.getSelectedValuesList()) {
			configuration.addActivity(s);
		}
		if(simpleConfiguration.isSelected()) {
			configuration.setMinerConfiguration(MINER_CONFIGURATION.SIMPLE);
		} else if (advancedConfiguration.isSelected()) {
			configuration.setMinerConfiguration(MINER_CONFIGURATION.ADVANCED);
		} else {
			configuration.setMinerConfiguration(MINER_CONFIGURATION.LOCALIZED_LOGS);
		}
		
		return configuration;
	}
}
