/**
 * 
 */
package prompt.multiperspective.externals.conformance;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

/**
 * @author aadrians
 * 
 */
public class PromptEvClassLogPetrinetConnectionFactory {
	public EvClassLogPetrinetConnection connect(UIPluginContext context, XLog log, PetrinetGraph net) {

		// list possible classifiers
		List<XEventClassifier> classList = new ArrayList<XEventClassifier>(log.getClassifiers());
		// add default classifiers
		if (!classList.contains(XLogInfoImpl.RESOURCE_CLASSIFIER)) {
			classList.add(0, XLogInfoImpl.RESOURCE_CLASSIFIER);
		}
		if (!classList.contains(XLogInfoImpl.NAME_CLASSIFIER)) {
			classList.add(0, XLogInfoImpl.NAME_CLASSIFIER);
		}
		if (!classList.contains(XLogInfoImpl.STANDARD_CLASSIFIER)) {
			classList.add(0, XLogInfoImpl.STANDARD_CLASSIFIER);
		}

		Object[] availableEventClass = classList.toArray(new Object[classList.size()]);
		EvClassLogPetrinetConnection con = null;
		int selection = 0;
		// build and show the UI to make the mapping
		PromptEvClassLogPetrinetConnectionFactoryUI ui = getUI(log, net, availableEventClass);
		
		do {
			InteractionResult result = null;
			
			// we must show the mapping UI only if there are uncertain automatic
			// choices
			if (ui.getRowCounter() > 1)
				result = context.showWizard("Mapping Petrinet - Event Class of Log", true, true, ui);
			else
				result = InteractionResult.FINISHED;

			// create the connection or not according to the button pressed in the UI
			if (result == InteractionResult.FINISHED) {
				// check if all event classes are mapped
				XLogInfo summary = XLogInfoFactory.createLogInfo(log, ui.getSelectedClassifier());
				XEventClasses eventClasses = summary.getEventClasses();
				Collection<XEventClass> colEventClasses = new HashSet<XEventClass>(eventClasses.getClasses());
				colEventClasses.removeAll(ui.getMap().values());
				
				// it is possible that the net does not contain some activities
				// in the log, causing an incomplete mapping and switching the
				// mapping gui event if there is nothing the user can do
				Iterator<XEventClass> iter = colEventClasses.iterator();
				while (iter.hasNext()) {
					XEventClass cl = iter.next();
					for (Transition t : net.getTransitions()){
						if (t.getLabel().compareTo(cl.getId()) == 0){
							break;
						}
					}
					iter.remove();
				}

				if (colEventClasses.size() > 0) {
					CheckMappingGUI cg = new CheckMappingGUI(colEventClasses);
					
					selection = cg.getSelection();

					if (selection == 0) {

						// mapping is finished, create connection
						con = new EvClassLogPetrinetConnection("Connection between " + net.getLabel() + " and "
								+ XConceptExtension.instance().extractName(log), net, log, ui.getSelectedClassifier(),
								ui.getMap());
						
					}
				} else {
					selection = 0;
					con = new EvClassLogPetrinetConnection("Connection between " + net.getLabel() + " and "
							+ XConceptExtension.instance().extractName(log), net, log, ui.getSelectedClassifier(),
							ui.getMap());
				}
			} else {
				return null;
			}
		} while (selection != 0);

		// return the connection (or null if the connection hasn't been created)
		return con;

	}
	
	/**
	 * Generate the UI to input user configuration
	 * 
	 * @param log event log
	 * @param net Petri Network
	 * @param availableEventClass 
	 * @return
	 */
	public PromptEvClassLogPetrinetConnectionFactoryUI getUI(XLog log, PetrinetGraph net, Object [] availableEventClass){
		return new PromptEvClassLogPetrinetConnectionFactoryUI(log, net, availableEventClass);
	}
}
