package prompt.multiperspective.externals.decisionminer;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.Connection;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.annotations.ConnectionObjectFactory;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.PluginExecutionResult;
import org.processmining.framework.plugin.PluginParameterBinding;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.util.Pair;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.models.FunctionEstimator.DecisionTreeFunctionEstimator;
import org.processmining.models.FunctionEstimator.FunctionEstimator;
import org.processmining.models.FunctionEstimator.Type;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.ResetInhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.ResetNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithDataFactory;
import org.processmining.models.guards.Expression;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.astar.petrinet.PetrinetReplayerWithoutILP;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayer.PNLogReplayer;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParam;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinetwithdata.decisionminer.ControlFlowAlignmentConnection;
import org.processmining.plugins.petrinetwithdata.decisionminer.TraceProcessor;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

import prompt.multiperspective.externals.conformance.PromptEvClassLogPetrinetConnectionFactory;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.google.common.util.concurrent.AtomicLongMap;

import csplugins.id.mapping.ui.CheckComboBox;

public class CustomDecisionMiner {
	
	private static ArrayList<String> booleanValues=new ArrayList<String>();
	private static JCheckBox pruneBox=SlickerFactory.instance().createCheckBox("Prune tree", true);
	private static JCheckBox binaryBox=SlickerFactory.instance().createCheckBox("Use Binary Conditions", false);
	private static NiceIntegerSlider instances4Leaf=SlickerFactory.instance().createNiceIntegerSlider("(In permil)", 1, 500, 20, Orientation.HORIZONTAL);
	//private static JCheckBox useDaikonBox=SlickerFactory.instance().createCheckBox("Use Invariant Detector (Daikon)", false);
	private static NiceIntegerSlider percentageOfWrite=SlickerFactory.instance().createNiceIntegerSlider("(In Percent)", 50, 100, 66, Orientation.HORIZONTAL);
	private static JCheckBox removeAttribInNoGuardBox=SlickerFactory.instance().createCheckBox("Remove Attributes that Occur in No Guard", true);
	private static NiceDoubleSlider fitnessThresholdSlider=null;
	
	static
	{
		booleanValues.add("T");
		booleanValues.add("F");
	}

	public static Object[] plugin(UIPluginContext context, PetrinetGraph net, XLog log, boolean setParameters) 
		throws Exception
	{
		int maxConcurrentThreads=Runtime.getRuntime().availableProcessors();
		ThreadPoolExecutor pool = new ThreadPoolExecutor(maxConcurrentThreads, maxConcurrentThreads, 60, TimeUnit.SECONDS,new LinkedBlockingQueue<Runnable>());
		pool.allowCoreThreadTimeOut(true);
		
		ControlFlowAlignmentConnection connection;
		PNRepResult input = null;
		try {
			connection = context.getConnectionManager().getFirstConnection(ControlFlowAlignmentConnection.class, context, net, log);
			input = connection.getObjectWithRole(ControlFlowAlignmentConnection.PNREPRESULT);
		} catch (ConnectionCannotBeObtained e1) {
			try {
				TransEvClassMapping mapping=createMapping(context, net, log,setParameters);
				CostBasedCompleteParam parameters=createParameters(log, mapping.getDummyEventClass(), net, context,setParameters, mapping.getEventClassifier());
				if (net instanceof InhibitorNet) {
					input=new PNLogReplayer().replayLog(context, (InhibitorNet)net, log, mapping, new PetrinetReplayerWithoutILP(), parameters);
				} else if (net instanceof Petrinet){
					System.out.println("context is : " + context);
					input=new PNLogReplayer().replayLog(context, (Petrinet)net, log, mapping, new PetrinetReplayerWithoutILP(), parameters);
				} else if (net instanceof ResetInhibitorNet) {
					input=new PNLogReplayer().replayLog(context, (ResetInhibitorNet)net, log, mapping, new PetrinetReplayerWithoutILP(), parameters);
				} else if (net instanceof ResetNet) {
					input=new PNLogReplayer().replayLog(context, (ResetNet)net, log, mapping, new PetrinetReplayerWithoutILP(), parameters);
				} else {
					throw(new IllegalArgumentException("The net is of a form that was not expected"));
				}
				context.getConnectionManager().addConnection(new ControlFlowAlignmentConnection("Control-Flow Alignment Connection",net,log,input));
			} catch (ConnectionCannotBeObtained e) {
				e.printStackTrace();
			}
		}

		Map<String, Type> classTypes=extractAttributeInformation(log);
		Map<Place, FunctionEstimator> estimators=new HashMap<Place, FunctionEstimator>();
//		Map<Transition, Integer> numberOfExecutions=Collections.synchronizedMap(new HashMap<Transition, Integer>());
		AtomicLongMap<Transition> numberOfExecutions = AtomicLongMap.create();
//		Map<Transition, Map<String,Integer>> numberOfWritesPerTransition=
//			Collections.synchronizedMap(new HashMap<Transition, Map<String,Integer>>());
		Map<Transition, AtomicLongMap<String>> numberOfWritesPerTransition = new HashMap<Transition, AtomicLongMap<String>>();
		for(Transition trans : net.getTransitions())
		{
			numberOfExecutions.put(trans, 0);
//			numberOfWritesPerTransition.put(trans, Collections.synchronizedMap(new HashMap<String, Integer>()));
			numberOfWritesPerTransition.put(trans, AtomicLongMap.<String>create());
		}
		
		Double initialFitness=((Double)input.getInfo().get(PNRepResult.TRACEFITNESS));
		fitnessThresholdSlider=SlickerFactory.instance().createNiceDoubleSlider("", 0.5, 1, initialFitness, Orientation.HORIZONTAL);
		
		if (setParameters && showDecisionTreePanel(estimators,context,classTypes)) {
			context.getFutureResult(0).cancel(true);
			return null;
		}			
	
		for(Place place : net.getPlaces())
		{
			if (net.getOutEdges(place).size()<2)
				continue;
			Transition outputValues[]=new Transition[net.getOutEdges(place).size()];
			int index=0;
			for(PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> arc : net.getOutEdges(place))
				outputValues[index++]=(Transition) arc.getTarget();
			Map<String, Set<String>> literalValues=getLiteralValuesMap(log);
			FunctionEstimator f;
			//if (useDaikonBox.isSelected())
			//	f=new DaikonDecisionTreeFunctionEstimator(classTypes, literalValues, outputValues, place.getLabel(), 1000);
			//else
				f=new DecisionTreeFunctionEstimator(classTypes, literalValues, outputValues, place.getLabel(), 1000);
			estimators.put(place, f);
		}
		
		final Progress progress=context.getProgress();
		progress.setMaximum(log.size()+2*estimators.entrySet().size());
		progress.setValue(0);
		context.log("Processing the log traces...");
		
		int traceSkipped=0;
		int totalNumTraces=0;
		for(SyncReplayResult alignment : input)
		{
			totalNumTraces+=alignment.getTraceIndex().size();
			if (alignment.getInfo().get(PNRepResult.TRACEFITNESS).floatValue() >= fitnessThresholdSlider.getValue())
				for(Integer index : alignment.getTraceIndex())
					pool.execute(new TraceProcessor(
							net,
							log.get(index),
							estimators,
							alignment,
							numberOfExecutions,
							numberOfWritesPerTransition,
							progress));
			else
				traceSkipped+=alignment.getTraceIndex().size();
		}
		context.log("Skipped "+traceSkipped+" low-fitting traces out of "+totalNumTraces);
		pool.shutdown();
		pool.awaitTermination(1, TimeUnit.DAYS);
		
		for(Place place : net.getPlaces())
		{
			FunctionEstimator f = estimators.get(place);
			if (f!=null && f instanceof DecisionTreeFunctionEstimator)
			{
				int numInstances=((DecisionTreeFunctionEstimator) f).getNumInstances();
				((DecisionTreeFunctionEstimator) f).setMinNumObj((int) (numInstances*(instances4Leaf.getValue()/1000F)));
				((DecisionTreeFunctionEstimator) f).setUnpruned(!pruneBox.isSelected());
				((DecisionTreeFunctionEstimator) f).setBinarySplit(binaryBox.isSelected());
			}
		}		
				
		final PetriNetWithDataFactory factory=new PetriNetWithDataFactory(net, net.getLabel());
		final PetriNetWithData retValue=factory.getRetValue();
		for(Entry<String, Type> entry : classTypes.entrySet())
		{
			Class classType=null;
			switch(entry.getValue())
			{
				case BOOLEAN :
					classType=Boolean.class;
					break;
				case CONTINUOS :
					classType=Double.class;
					break;
				case DISCRETE :
					classType=Long.class;
					break;
				case LITERAL :
					classType=String.class;
					break;
				case TIMESTAMP :
					classType=Date.class;
					break;
				default :
					break;
				
			}
			retValue.addVariable(entry.getKey(), classType, null, null);
		}
		
	
		Locale defaultLocale=Locale.getDefault();
		Locale.setDefault(Locale.US);
		
		double sumFScores=0;
		int numEstimators=0;

		for(Entry<Place, FunctionEstimator> entry2 : estimators.entrySet())
		{
			context.log("Generating the conditions for the decision point at place "+entry2.getKey().getLabel());	
			final FunctionEstimator f=entry2.getValue();

			Map<Object, Pair<Expression, Double>> estimationTransitionExpression = null;
			try {
				estimationTransitionExpression = f.getEstimation(null);
				System.out.println("Place: "+entry2.getKey().getLabel());
				System.out.println(f);
				if (f instanceof DecisionTreeFunctionEstimator && !estimationTransitionExpression.isEmpty())
				{
					double singleFScore=((DecisionTreeFunctionEstimator) f).computeFMeasure();
					context.log("F score: "+singleFScore);
					sumFScores+=singleFScore;
					numEstimators++;
				}
			} catch (Exception e) {
				e.printStackTrace();
				context.log(e.getMessage());
			}
			if (estimationTransitionExpression!=null)
				for(Entry<Object, Pair<Expression, Double>> entry : estimationTransitionExpression.entrySet())
				{
					Transition transitionInPNWithoutData=(Transition) entry.getKey();
					Transition transitionInPNWithData = factory.getTransMapping().get(transitionInPNWithoutData);
					Pair<Expression, Double> value=entry.getValue();
					retValue.setGuard(transitionInPNWithData, value.getFirst(),value.getSecond());
				}

			progress.inc();
			progress.inc();
	

		}
		Locale.setDefault(defaultLocale);
		if(sumFScores>0)
		{
			context.log("Average F Score: "+sumFScores/numEstimators);
		}

		for(Transition transitionInPNWithoutData : net.getTransitions())
		{
			PNWDTransition transitionInPNWithData = (PNWDTransition) factory.getTransMapping().get(transitionInPNWithoutData);

			//Set the read operations
			if (transitionInPNWithData.getGuard()!=null)
			{
				Enumeration enumer=transitionInPNWithData.getGuard().findVariables();
				while(enumer.hasMoreElements())
				{
					String varName=(String) enumer.nextElement();
					retValue.assignReadOperation(transitionInPNWithData, retValue.getVariable(varName));
				}
			}


			//Set the write operations
			if (!transitionInPNWithData.isInvisible())
			{
				long numberOfExecution = numberOfExecutions.get(transitionInPNWithoutData);
				for (Entry<String, Long> entry3 : numberOfWritesPerTransition.get(transitionInPNWithoutData).asMap().entrySet()) {
//				for(Entry<String,Integer> entry3 : numberOfWritesPerTransition.get(transitionInPNWithoutData).entrySet())
//				{
					if (entry3.getValue() > (numberOfExecution * percentageOfWrite.getValue())/100)
					{
						DataElement dataElem=retValue.getVariable(entry3.getKey());
						if (dataElem!=null)
							retValue.assignWriteOperation(transitionInPNWithData, dataElem);
					}
				}
			}
		}

		if (removeAttribInNoGuardBox.isSelected())
			retValue.removeAllVariablesNotInGuard();
		
		Marking[] markings=factory.cloneInitialAndFinalConnection(context);
		
		//retValue=new DiscoveredPetriNetWithData(retValue,visTree);
			
		return new Object[] { retValue, markings[0], markings[1]};	

	}
			


	private static TransEvClassMapping createMapping(UIPluginContext context, PetrinetGraph net, XLog log, boolean setParameters) throws ConnectionCannotBeObtained {
//		EvClassLogPetrinetConnection connection2=context.getConnectionManager().getFirstConnection(EvClassLogPetrinetConnection.class, context, net, log);
		
		
		PromptEvClassLogPetrinetConnectionFactory connector = new PromptEvClassLogPetrinetConnectionFactory();
		EvClassLogPetrinetConnection connection2 = connector.connect(context, log, net);
		TransEvClassMapping mapping = connection2.getObjectWithRole(EvClassLogPetrinetConnection.TRANS2EVCLASSMAPPING);
		
		Set<Transition> unmappedTrans = new HashSet<Transition>();
		for (Entry<Transition, XEventClass> entry : mapping.entrySet()) {
			if (entry.getValue().equals(mapping.getDummyEventClass())) {
				if (!entry.getKey().isInvisible()) {
					unmappedTrans.add(entry.getKey());
				}
			}
		}
		
		if (!unmappedTrans.isEmpty()) {
			JList list = new JList(unmappedTrans.toArray());
			JPanel panel = new JPanel();
			
			BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
			panel.setLayout(layout);
			panel.add(new JLabel("The following transitions are not mapped to any event class:"));

			JScrollPane sp = new JScrollPane(list);
			panel.add(sp);
			panel.add(new JLabel("Do you want to consider these transitions as invisible (unlogged activities)?"));

			Object[] options = { "Yes, set them to invisible", "No, keep them as they are" };

			if (!setParameters || 0 == JOptionPane.showOptionDialog(null, panel, "Configure transition visibility",
					JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[0])) {
				for (Transition t : unmappedTrans) {
					t.setInvisible(true);
				}
			}
			;
		}
		return mapping;
	}

	private static boolean showDecisionTreePanel(Map<Place, FunctionEstimator> estimators, UIPluginContext context, Map<String, Type> classTypes) 
	{
		ProMPropertiesPanel panel=new ProMPropertiesPanel("");
		String[] attributes=classTypes.keySet().toArray(new String[0]);
		Arrays.sort(attributes);
		CheckComboBox attributeCCBox=new CheckComboBox(attributes,true);
		attributeCCBox.setPreferredSize(new Dimension(300,50));
		attributeCCBox.setMaximumSize(new Dimension(300,50));
		panel.addProperty("Attributes to Consider for mining:", attributeCCBox);
		panel.addProperty("",SlickerFactory.instance().createLabel("Minimal Number of Occurrences of Write Operations to be mined")); 
		panel.addProperty("", percentageOfWrite);
		panel.addProperty("", removeAttribInNoGuardBox);
		panel.addProperty("",SlickerFactory.instance().createLabel("Minimal Fitness for a trace to be considered"));		
		panel.addProperty("", fitnessThresholdSlider);

			panel.addProperty("",SlickerFactory.instance().createLabel("Minimal Numbers of Instances per leaf"));
			panel.addProperty("", instances4Leaf);
			panel.addProperty("", pruneBox);
			panel.addProperty("", binaryBox);		
//			panel.addProperty("", useDaikonBox);			

		InteractionResult result = context.showConfiguration("Configuration", panel);
		if (result == InteractionResult.CANCEL) {
			return true;
		}
		
		if (attributeCCBox.getSelectedItems()!=null)
		{
			Iterator<Entry<String, Type>> iter=classTypes.entrySet().iterator();
			while(iter.hasNext())
			{
				if (!attributeCCBox.getSelectedItems().contains(iter.next().getKey()))
					iter.remove();
			}
		}
		else
			return true;
		return false;
	}

	private static CostBasedCompleteParam createParameters(XLog log, XEventClass eventClass, PetrinetGraph net, PluginContext context, 
			boolean setParameters, XEventClassifier classifier) throws ConnectionCannotBeObtained
	{
		XLogInfo logInfo=XLogInfoFactory.createLogInfo(log, classifier);
		CostBasedCompleteParam parameter = new CostBasedCompleteParam(logInfo.getEventClasses().getClasses(), eventClass, net.getTransitions(), 1, 1);
		parameter.setGUIMode(false);
		parameter.setCreateConn(false);
		
		Marking initMarking = context.tryToFindOrConstructFirstObject(Marking.class, InitialMarkingConnection.class, InitialMarkingConnection.MARKING, net);
		if (initMarking.isEmpty() && setParameters)
		{
			createMarking((UIPluginContext) context, net, InitialMarkingConnection.class);
			try {
				initMarking = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, net).
						getObjectWithRole(FinalMarkingConnection.MARKING);
			} catch (Exception e) {
				initMarking=new Marking();
			}
		}
		parameter.setInitialMarking(initMarking);
		
		Marking finalMarking = context.tryToFindOrConstructFirstObject(Marking.class, FinalMarkingConnection.class, FinalMarkingConnection.MARKING, net);
		if (finalMarking.isEmpty() && setParameters)
		{
			createMarking((UIPluginContext) context, net, FinalMarkingConnection.class);
			finalMarking = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, net).
					getObjectWithRole(FinalMarkingConnection.MARKING);
		}
		parameter.setFinalMarkings(new Marking[] {finalMarking});
		parameter.setMaxNumOfStates(200000);
		return parameter;
	}
	
	private static boolean createMarking(UIPluginContext context, PetrinetGraph net, Class<? extends Connection> classType) {
		boolean result = false;
		Collection<Pair<Integer, PluginParameterBinding>> plugins = context.getPluginManager().find(
				ConnectionObjectFactory.class, classType, context.getClass(), true, false, false, net.getClass());
		PluginContext c2 = context.createChildContext("Creating connection of Type " + classType);
		Pair<Integer, PluginParameterBinding> pair = plugins.iterator().next();
		PluginParameterBinding binding = pair.getSecond();
		try {
			PluginExecutionResult pluginResult = binding.invoke(c2, net);
			pluginResult.synchronize();
			context.getProvidedObjectManager().createProvidedObjects(c2); // push the objects to main context
			result = true;
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			c2.getParentContext().deleteChild(c2);
		}
		
		return result;
	}	

	
	public static Map<String, Set<String>> getLiteralValuesMap(XLog log) {	
		return getLiteralValuesMap(log, false);
	}
	
	
	public static Map<String, Set<String>> getLiteralValuesMap(XLog log, boolean forgetTraceAttribute) {
		
		Map<String, Set<String>> retValue=new HashMap<String, Set<String>>();
		
		for(XTrace trace : log) {
			
			if (!forgetTraceAttribute) {
				
				for(XAttribute attributeEntry : trace.getAttributes().values()) {
					
					if (attributeEntry instanceof XAttributeLiteral) {
						
						String value = ((XAttributeLiteral)attributeEntry).getValue();
						Set<String> literalValues = retValue.get(attributeEntry.getKey());
						
						if (literalValues == null) {
							literalValues = new HashSet<String>();
							retValue.put(attributeEntry.getKey(), literalValues);
						}
						
						literalValues.add(value);
					}
				}
			}
			
			for(XEvent event : trace) {
				
				for(XAttribute attributeEntry : event.getAttributes().values()) {
					
					if (attributeEntry instanceof XAttributeLiteral) {
						
						String value = ((XAttributeLiteral)attributeEntry).getValue();
						Set<String> literalValues = retValue.get(attributeEntry.getKey());
						
						if (literalValues == null) {
							literalValues=new HashSet<String>();
							retValue.put(attributeEntry.getKey(), literalValues);
						}
						
						literalValues.add(value);
					}
				}
			}
		}
		
		return retValue;
	}

	
	public static Map<String, Type> extractAttributeInformation(XLog log) {
		return extractAttributeInformation(log, false);
	}

	
	public static Map<String, Type> extractAttributeInformation(XLog log, boolean forgetTraceAttribute) {
		HashMap<String, Type> retValue = new HashMap<String, Type>();
		for (XTrace trace : log) {
			
			if(!forgetTraceAttribute) {
				for(XAttribute attr : trace.getAttributes().values()) {
					Type classType = generateDataElement(attr);
					if (classType != null)
						retValue.put(attr.getKey(), classType);
				}
			}
			
			for (XEvent event : trace)
			{
				for(XAttribute attr : event.getAttributes().values())
				{
					if (!attr.getKey().contains(":"))
					{
						Type classType = generateDataElement(attr);
						if (classType != null)
							retValue.put(attr.getKey(), classType);
					}
				}
			}
			
		}
		
		return retValue;
	}
	
	
	private static Type generateDataElement(XAttribute xAttrib) {

		if (xAttrib instanceof XAttributeBoolean) {
			return Type.BOOLEAN;
		} else if (xAttrib instanceof XAttributeContinuous) {
			return Type.CONTINUOS;
		} else if (xAttrib instanceof XAttributeDiscrete) {
			return Type.DISCRETE;
		} else if (xAttrib instanceof XAttributeTimestamp) {
			return Type.TIMESTAMP;
		} else if (xAttrib instanceof XAttributeLiteral) {
			return Type.LITERAL;
		}
		
		return null;	
	}
	
}

