package prompt.multiperspective.plugins;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.loclog.utils.BPMNUtils;
import org.processmining.models.graphbased.directed.ContainableDirectedGraphElement;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagramImpl;
import org.processmining.models.graphbased.directed.bpmn.BPMNNode;
import org.processmining.models.graphbased.directed.bpmn.elements.SubProcess;
import org.processmining.models.graphbased.directed.bpmn.elements.Swimlane;
import org.processmining.models.graphbased.directed.bpmn.elements.SwimlaneType;

import prompt.multiperspective.utils.AuthorsInfo;
import prompt.multiperspective.utils.PluginNames;
import prompt.roles.data.RoleSet;


/**
 * 
 * @author Anna Kalenkova
 *
 */
public class HierarchicalRolesMiner {
	
	@Plugin(
		name = PluginNames.HIERARCHICAL_ROLES_MINER,
		parameterLabels = { "BPMN Model", "Event Log" },
		returnLabels = { "BPMN Model" },
		returnTypes = { BPMNDiagram.class },
		userAccessible = true
	)
	@UITopiaVariant(
		author = AuthorsInfo.AUTHOR,
		email = AuthorsInfo.EMAIL,
		affiliation = AuthorsInfo.AFFILIATION
	)
	public static BPMNDiagram plugin(UIPluginContext context, BPMNDiagram diagram, XLog log) {
		try {
			List<SubProcess> subprocessesToAnalyze = new ArrayList<SubProcess>();
			subprocessesToAnalyze.addAll(diagram.getSubProcesses());
			while(subprocessesToAnalyze.size() > 0) {
				SubProcess subprocess = subprocessesToAnalyze.get(0);
				boolean hasSubSubProcesses = false;
				for(ContainableDirectedGraphElement c : subprocess.getChildren()) {
					if ((c instanceof SubProcess) && (subprocessesToAnalyze.contains(c))) {
						hasSubSubProcesses = true;
					}
				}
				if(hasSubSubProcesses) {
					subprocessesToAnalyze.remove(0);
					subprocessesToAnalyze.add(subprocess);
					continue;
				}
				BPMNDiagram  subDiagram = new BPMNDiagramImpl(subprocess.getLabel());
				HashMap<ContainableDirectedGraphElement, ContainableDirectedGraphElement> map = 
						BPMNUtils.constructBPMNDiagramFromSubprocess(subDiagram, subprocess, diagram);
				RoleSet roles = null;
				try {
					roles = context.tryToFindOrConstructFirstNamedObject(RoleSet.class, PluginNames.SWIMLANE_DISCOVERY, null, null, subDiagram, log);
				} catch (NullPointerException ex) {
				}
					BPMNDiagram subModelWithSwimlanes = new BPMNDiagramImpl(diagram.getLabel());
				if ((roles != null) && (roles.size() > 1)) {
					subModelWithSwimlanes = context.tryToFindOrConstructFirstNamedObject(BPMNDiagram.class, PluginNames.DECORATE_BPMN_WITH_SWIMLANES, null, null, subDiagram, roles);
				}
				for(Swimlane lane : subModelWithSwimlanes.getSwimlanes()) {
					if(lane.getSwimlaneType().equals(SwimlaneType.LANE)) {
						Swimlane newLane = diagram.addSwimlane(lane.getLabel(), subprocess, SwimlaneType.LANE);
						for(ContainableDirectedGraphElement c : lane.getChildren()) {
							ContainableDirectedGraphElement child = map.get(c);
							subprocess.getChildren().remove(child);
							((BPMNNode)child).setParentSubprocess(null);
							newLane.addChild(child);
							((BPMNNode)child).setParentSwimlane(newLane);
							
						}
					}
				}
				subprocessesToAnalyze.remove(subprocess);
			}
		} catch(ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		return diagram;
	}
}
