package prompt.multiperspective.plugins;

import java.io.Serializable;

import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.framework.annotations.AuthoredType;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;

import prompt.roles.data.RoleSet;

/**
 * This class contains the main result of a multiperspective miner plugin run.
 * 
 * @author Andrea Burattin
 */
@AuthoredType(
	typeName = "Multiperspective Model",
	author = "Andrea Burattin",
	email = "PROMPT Project",
	affiliation = ""
)
public class MultiperspectiveModel implements Serializable {

	private static final long serialVersionUID = 6308117908700351938L;
	private BPMNDiagram diagram;
	private RoleSet roleSet;
	//private AdvancedSocialNetwork advancedSNA = null;
	private DataPetriNet dataPetriNet;
	private DataPetriNet dataPetriNetNoVars;
	
	/**
	 * Class main constructor
	 * 
	 * @param diagram the BPMN diagram
	 * @param roleSet the role set associated to the current role partitioning
	 */
	public MultiperspectiveModel(BPMNDiagram diagram, RoleSet roleSet) {
		this.diagram = diagram;
		this.roleSet = roleSet;
	}
	
	/**
	 * Class constructor
	 */
	public MultiperspectiveModel() {
		this(null, null);
	}
	
	/**
	 * 
	 * @return the diagram
	 */
	public BPMNDiagram getDiagram() {
		return diagram;
	}
	
	/**
	 * 
	 * @param diagram
	 */
	public void setDiagram(BPMNDiagram diagram) {
		this.diagram = diagram;
	}
	
	/**
	 * 
	 * @return the role set
	 */
	public RoleSet getRoleSet() {
		return roleSet;
	}
	
	/**
	 * 
	 * @param roleSet
	 */
	public void setRoleSet(RoleSet roleSet) {
		this.roleSet = roleSet;
	}
	
	/**
	 * 
	 * @return the data Petri net
	 */
	public DataPetriNet getDataPetriNet() {
		return dataPetriNet;
	}
	
	/**
	 * 
	 * @return the data Petri net
	 */
	public DataPetriNet getDataPetriNetWithoutVariables() {
		return dataPetriNetNoVars;
	}
	
	/**
	 * 
	 * @param dataPetriNet
	 */
	public void setDataPetriNetWithoutVariables(DataPetriNet dataPetriNetNoVars) {
		this.dataPetriNetNoVars = dataPetriNetNoVars;
	}
	
	/**
	 * 
	 * @param dataPetriNet
	 */
	public void setDataPetriNet(DataPetriNet dataPetriNet) {
		this.dataPetriNet = dataPetriNet;
	}
	
	/**
	 * 
	 * @return the advanced Social Network
	 */
	/*public AdvancedSocialNetwork getAdvancedSocialNetwork() {
		return advancedSNA;
	}*/
	
	/**
	 * 
	 * @param AdvancedSocialNetwork
	 */
	/*public void setAdvancedSocialNetwork(AdvancedSocialNetwork advancedSNA) {
		this.advancedSNA = advancedSNA;
	}*/
}
