package prompt.multiperspective.plugins.io;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.ZipOutputStream;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.datapetrinets.io.DataPetriNetExporter;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.bpmn.plugins.BpmnExportPlugin;

import prompt.multiperspective.plugins.MultiperspectiveModel;
import prompt.roles.data.RoleSet;

/**
 * This class is responsible of exporting multiperspecive models.
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Multiperspective Model (.mpm)",
	returnLabels = {},
	returnTypes = {},
	parameterLabels = { "Multiperspective Model", "File" },
	userAccessible = true
)
@UIExportPlugin(
	description = "Multiperspective Model (.mpm)",
	extension = "mpm"
)
public class MultiperspectiveModelExport {
	
	/**
	 * The actual export method
	 * 
	 * @param context
	 * @param model
	 * @param file
	 */
	@PluginVariant(
		requiredParameterLabels = { 0, 1 }
	)
	public void exportFile(UIPluginContext context, MultiperspectiveModel model, File file) {
		try {
			ZipOutputStream out = new ZipOutputStream(new FileOutputStream(file));
			
			// saving role set -------------------------------------------------
			File tempFile1 = File.createTempFile("roles", ".roleset");
			tempFile1.deleteOnExit();
			RoleSet.saveToFile(model.getRoleSet(), tempFile1);
			// add to zip
			MultiperspectiveModelIOUtils.saveToZip(out, tempFile1, MultiperspectiveModelIOUtils.ROLESET_FILENAME);
			
			// saving bpmn model -----------------------------------------------
			File tempFile2 = File.createTempFile("model", ".bpmn");
			tempFile2.deleteOnExit();
			BpmnExportPlugin exportBPMN = new BpmnExportPlugin();
			exportBPMN.export(context, model.getDiagram(), tempFile2);
			// add to zip
			MultiperspectiveModelIOUtils.saveToZip(out, tempFile2, MultiperspectiveModelIOUtils.BPMN_FILENAME);
			
			// saving data petri net -------------------------------------------
			File tempFile3 = File.createTempFile("data-petri-net", ".pn");
			tempFile3.deleteOnExit();
			DataPetriNetExporter exporter2 = new DataPetriNetExporter();
			exporter2.exportPetriNetToPNMLFile(context, model.getDataPetriNet(), tempFile3);
			// add to zip
			MultiperspectiveModelIOUtils.saveToZip(out, tempFile3, MultiperspectiveModelIOUtils.DATAPETRINET_FILENAME);
			
			// saving data petri net with no data ------------------------------
			File tempFile4 = File.createTempFile("data-petri-net-no-vars", ".pn");
			tempFile4.deleteOnExit();
			exporter2.exportPetriNetToPNMLFile(context, model.getDataPetriNetWithoutVariables(), tempFile4);
			// add to zip
			MultiperspectiveModelIOUtils.saveToZip(out, tempFile4, MultiperspectiveModelIOUtils.DATAPETRINET_NOVARS_FILENAME);
			
			// saving advanced social network ----------------------------------
			/*File tempFile5 = File.createTempFile("advanced-sna", ".gsmp");
			tempFile5.deleteOnExit();
			SocialNetworkExporter2 ASNExport = new SocialNetworkExporter2();
			ASNExport.write_gsmp(model.getAdvancedSocialNetwork(), tempFile5);
			// add to zip
			MultiperspectiveModelIOUtils.saveToZip(out, tempFile5, MultiperspectiveModelIOUtils.ADVANCED_SNA_FILENAME);*/
			
			out.close();
		
		} catch(IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
