package prompt.multiperspective.plugins.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * Some common stuff of Multiperspective Models IO.
 * 
 * @author Andrea Burattin
 */
public class MultiperspectiveModelIOUtils {

	/**
	 * BPMN model filename inside the ZIP file
	 */
	public static final String BPMN_FILENAME = "model.bpmn";
	
	/**
	 * Roles set model filename inside the ZIP file
	 */
	public static final String ROLESET_FILENAME = "roles.roleset";
	
	/**
	 * Data Petri net model filename inside the ZIP file
	 */
	public static final String DATAPETRINET_FILENAME = "data-petri-net.pnml";

	/**
	 * Data Petri without variables net model filename inside the ZIP file
	 */
	public static final String DATAPETRINET_NOVARS_FILENAME = "data-petri-net-no-vars.pnml";

	/**
	 * Advanced Social Network filename inside the ZIP file
	 */
	public static final String ADVANCED_SNA_FILENAME = "advanced-sna.gsmp";
	
	/**
	 * This method exports a file into a zip container
	 * 
	 * @param zipStreamContainer the zip container
	 * @param resource the file to export
	 * @param zipFileName the name of the file within the zip file
	 * @throws IOException
	 */
	public static void saveToZip(ZipOutputStream zipStreamContainer, File resource, String zipFileName) throws IOException {
		FileInputStream fis = new FileInputStream(resource.getAbsolutePath());
		zipStreamContainer.putNextEntry(new ZipEntry(zipFileName));
		int len;
		byte[] buf = new byte[1024];
		while ((len = fis.read(buf)) > 0) {
			zipStreamContainer.write(buf, 0, len);
		}
		zipStreamContainer.closeEntry();
		fis.close();
	}
}
