package prompt.multiperspective.utils;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import org.apache.commons.io.IOUtils;

import prompt.multiperspective.configuration.MultiperspectiveConfiguration;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.jgoodies.forms.builder.DefaultFormBuilder;

/**
 * A collection of user interface utilities
 * 
 * @author Andrea Burattin
 */
public class GUIUtils {

	public static Color DEFAULT_COLOR = Color.gray;
	public static Color FOCUSED_COLOR = Color.black;

	/**
	 * 
	 * @param field
	 */
	public static void configureJComboBox(JComboBox<?> field) {
		field.setBackground(Color.lightGray);
		field.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createLineBorder(Color.gray),
				BorderFactory.createEmptyBorder(3, 3, 3, 3)));
	}
	
	/**
	 * 
	 * @param subtitle
	 * @return
	 */
	public static JPanel getSubtitle(String subtitle) {
		JLabel l = new JLabel(subtitle);
		l.setFont(l.getFont().deriveFont(14f));
		l.setForeground(Color.lightGray);
		
		JPanel p = new RoundedPanel(15);
		p.setBackground(Color.darkGray);
		p.setLayout(new BorderLayout());
		p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		p.setMaximumSize(new Dimension(780, 40));
		p.add(l, BorderLayout.CENTER);
		
		return p;
	}

	/**
	 * This method configure a separator build with the
	 * {@link DefaultFormBuilder#appendSeparator(String)} method.
	 * 
	 * @param appendSeparator
	 */
	public static void configureSeparator(JComponent appendSeparator) {
		JLabel lab = (JLabel) appendSeparator.getComponent(0);
		lab.setForeground(Color.white);
		lab.setFont(lab.getFont().deriveFont(14f));
		JSeparator sep = (JSeparator) appendSeparator.getComponent(1);
		sep.setForeground(Color.white);
	}

	/**
	 * This method configure a separator build with the
	 * {@link DefaultFormBuilder#appendSeparator(String)} method. This method
	 * hides the horizontal line.
	 * 
	 * @param appendSeparator
	 */
	public static void configureSeparatorWithoutLine(JComponent appendSeparator) {
		JLabel lab = (JLabel) appendSeparator.getComponent(0);
		lab.setForeground(Color.white);
		lab.setFont(lab.getFont().deriveFont(14f));
		JSeparator sep = (JSeparator) appendSeparator.getComponent(1);
		sep.setVisible(false);
	}
	
	/**
	 * This method creates a new JLabel correctly formatted
	 * 
	 * @param s the string to be inserted into the label
	 * @return a component with the required widget
	 */
	public static JLabel prepareLabel(String s) {
		return prepareLabel(s, SwingConstants.LEFT);
	}
	
	/**
	 * This method creates a new JLabel correctly formatted
	 * 
	 * @param s the string to be inserted into the label
	 * @param alignment the alignment
	 * @return a component with the required widget
	 */
	public static JLabel prepareLabel(String s, int alignment) {
		JLabel l = SlickerFactory.instance().createLabel(s);
		l.setHorizontalAlignment(alignment);
		l.setForeground(new Color(40,40,40));
		return l;
	}
	
	/**
	 * This method creates a new JLabel correctly formatted in order to be a
	 * frame title
	 * 
	 * @param s the string to be inserted into the label
	 * @return a component with the required widget
	 */
	public static JLabel prepareTitle(String s) {
		JLabel labelTitle = SlickerFactory.instance().createLabel(s);
		labelTitle.setFont(labelTitle.getFont().deriveFont(Font.BOLD).deriveFont(16f));
		labelTitle.setForeground(Color.darkGray);
		labelTitle.setBorder(new EmptyBorder(5, 0, 10, 0));
		return labelTitle;
	}
	
	/**
	 * This method can be used to generate a {@link JLabel} initialized with the
	 * content of n HTML file.
	 * 
	 * @param htmlFile the path of the HTML file
	 * @return a new {@link JLabel}
	 */
	public static JLabel fromHTMLToJLabel(String htmlFile) {
		String PROMPT_LOGO = GUIUtils.class.getResource("/prompt/multiperspective/resources/img/prompt-logo.png" ).toString();
		InputStream in = GUIUtils.class.getResourceAsStream("/prompt/multiperspective/resources/html/" + htmlFile);
		JLabel l = new JLabel();
		try {
			String text = IOUtils.toString(in, "UTF-8");
			text = text.replaceAll("PROMPT_LOGO", PROMPT_LOGO);
			l.setText(text);
			return l;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * This method can be used to load an icon from a filename
	 * 
	 * @param filename the filename
	 * @return the loaded icon
	 */
	public static ImageIcon loadIcon(String filename) {
		ImageIcon icon = null;
		try {
			InputStream in = MultiperspectiveConfiguration.class.getResourceAsStream(filename);
			icon = new ImageIcon(ImageIO.read(in));
		} catch (IOException e) {
			e.printStackTrace();
		}
		return icon;
	}
}
